#include <_Custom_Charts/Indicadores/Indicadores.mqh>
#include <_Custom_Operacional/Enums.mqh>
#include <_Custom_MetaInfo/Ativo.mqh>

class Strategy{
private:
    ENUM_STRATEGY_SIGN ultimoSinal;

public:
    Strategy(){};
    Strategy(const Strategy& stg){};
    ~Strategy(){};

    //Método genérico a ser implementado pelas classes filhas da definição do indicador da strategy
    virtual void set();
    //Define a relação do mercado com os buffers do indicador
    virtual void setBuffersRelation();
    //Retorna uma relação de buffer a partir de um preço especificado
    ENUM_MARKET_TO_BUFFER_RELATION getInitialBufferRelation(double valorBuffer);
    //Retorna uma relação de buffer a partir de um preço especificado e uma relação já definida
    ENUM_MARKET_TO_BUFFER_RELATION getBufferRelation(double valorBuffer, ENUM_MARKET_TO_BUFFER_RELATION relacaoAtual);
    //Método genérico a ser implementado pelas classes filhas para retorno da indicação atual da strategy
    virtual ENUM_STRATEGY_SIGN getSign();
    //Método genérico para setar o valor de relação dos buffers para não relacionados
    virtual void setUnrelatedBuffers();
    //Reinicia a strategy
    void restartStrategy();
    //Retorna o nível de venda
    ENUM_STRATEGY_SIGN getSellSignLevel(int indicacoes);
    //Retorna o nível de compra
    ENUM_STRATEGY_SIGN getBuySignLevel(int indicacoes);
    //Retorna se é um sinal de compra, independente da força
    bool isBuy(ENUM_STRATEGY_SIGN sinal);
    //Retorna se é um sinal de venda, independente da força
    bool isSell(ENUM_STRATEGY_SIGN sinal);
    //Define o ultimo sinal da estratégia
    void setUltimoSinal(ENUM_STRATEGY_SIGN sinal) { this.ultimoSinal = sinal; }
    //Retorna o ultimo sinal
    ENUM_STRATEGY_SIGN getUltimoSinal() { return this.ultimoSinal; }

    //Classe para controle de ativos
    Ativo ativo;
};

ENUM_MARKET_TO_BUFFER_RELATION Strategy::getInitialBufferRelation(double valorBuffer){
    double ultimoTick = this.ativo.getLast();
    if(ultimoTick > valorBuffer)
        return ABOVE;
    else if(ultimoTick < valorBuffer)
        return UNDER;
    else
        return UNRELATED_TO_BUFFER;
}

ENUM_MARKET_TO_BUFFER_RELATION Strategy::getBufferRelation(double valorBuffer, ENUM_MARKET_TO_BUFFER_RELATION relacaoAtual){
    ENUM_MARKET_TO_BUFFER_RELATION novaRelacao = this.getInitialBufferRelation(valorBuffer);
    if(relacaoAtual != novaRelacao){
        if((relacaoAtual == ABOVE || relacaoAtual == CROSSED_UP) && novaRelacao == UNDER)
            return CROSSED_DOWN;
        else if((relacaoAtual == UNDER || relacaoAtual == CROSSED_DOWN) && novaRelacao == ABOVE)
            return CROSSED_UP;
    }
    return novaRelacao;
}

ENUM_STRATEGY_SIGN Strategy::getSellSignLevel(int indicacoes){
    if(indicacoes == 0)
        return NONE_SIGN;

    if(indicacoes > 2)
        return HEAVY_SELL;
    else if(indicacoes < 2)
        return LIGHT_SELL;
    else
        return AVERAGE_SELL;
}

ENUM_STRATEGY_SIGN Strategy::getBuySignLevel(int indicacoes){
    if(indicacoes == 0)
        return NONE_SIGN;

    if(indicacoes > 2)
        return HEAVY_BUY;
    else if(indicacoes < 2)
        return LIGHT_BUY;
    else
        return AVERAGE_BUY;
}

bool Strategy::isBuy(ENUM_STRATEGY_SIGN sinal){
    if(sinal == HEAVY_BUY || sinal == AVERAGE_BUY || sinal == LIGHT_BUY)
        return true;
    return false;
}

bool Strategy::isSell(ENUM_STRATEGY_SIGN sinal){
    if(sinal == HEAVY_SELL || sinal == AVERAGE_SELL || sinal == LIGHT_SELL)
        return true;
    return false;
}